# SPDX-FileCopyrightText: 2026 The glTF-Blender-IO authors
#
# SPDX-License-Identifier: Apache-2.0


from ..com import conversion as gltf2_blender_conversion
from .attribute_utils import extract_attribute_data
import numpy as np


def gather_point_cloud(blender_pointcloud, export_settings):

    primitives = []

    # Do not export if we don't export loose points
    if not export_settings['gltf_loose_points']:
        return []

    # Position
    locs = np.empty(
        len(blender_pointcloud.attributes['position'].data) * 3, dtype=np.float32)
    position_attribute = gltf2_blender_conversion.get_attribute(
        blender_pointcloud.attributes, 'position', 'FLOAT_VECTOR', 'POINT')
    source = position_attribute.data if position_attribute else None
    foreach_attribute = 'vector'
    if source:
        source.foreach_get(foreach_attribute, locs)
    locs = locs.reshape(len(blender_pointcloud.attributes['position'].data), 3)

    # Radius
    radius = np.empty(
        len(blender_pointcloud.attributes['radius'].data), dtype=np.float32)
    radius_attribute = gltf2_blender_conversion.get_attribute(
        blender_pointcloud.attributes, 'radius', 'FLOAT', 'POINT')
    source = radius_attribute.data if radius_attribute else None
    foreach_attribute = 'value'
    if source:
        source.foreach_get(foreach_attribute, radius)
    radius = radius.reshape(len(blender_pointcloud.attributes['radius'].data))

    # Get any other attributes that may be present, starting with an underscore
    custom_attributes = __get_custom_attributes(blender_pointcloud, export_settings)

    custom_attributes['POSITION'] = {
        'data': locs,
        'data_type': gltf2_blender_conversion.get_data_type('FLOAT_VECTOR'),
        'component_type': gltf2_blender_conversion.get_component_type('FLOAT_VECTOR')
    }
    custom_attributes['_RADIUS'] = {
        'data': radius,
        'data_type': gltf2_blender_conversion.get_data_type('FLOAT'),
        'component_type': gltf2_blender_conversion.get_component_type('FLOAT')
    }

    primitives.append({
        'attributes': custom_attributes,
        'mode': 0,  # POINTS
        'material': 0,  # TODOPC
        'uvmap_attributes_index': {}
    })

    export_settings['log'].info(
        'Point Cloud Primitives created: %d' % len(primitives))

    return primitives


def __get_custom_attributes(blender_pointcloud, export_settings):
    custom_attributes = {}
    for attribute in blender_pointcloud.attributes:
        if attribute.domain != 'POINT':
            continue
        if attribute.name in ['position', 'radius']:
            continue
        if not attribute.name.startswith("_"):
            continue

        len_attr = gltf2_blender_conversion.get_data_length(
            attribute.data_type)

        data = extract_attribute_data(
            attribute,
            len(attribute.data),
            np.float32,
            attribute.data_type,
            attribute.domain,
            len_attr,
            export_settings=export_settings
        )

        if len_attr > 1:
            data = data.reshape(-1, len_attr)
        custom_attributes[attribute.name] = {
            'data': data,
            'data_type': gltf2_blender_conversion.get_data_type(attribute.data_type),
            'component_type': gltf2_blender_conversion.get_component_type(attribute.data_type)
        }
    return custom_attributes



import bpy
bpy.context.scene.render.resolution_x = 3840
bpy.context.scene.render.resolution_y = 2160
bpy.context.scene.render.resolution_percentage = 100
bpy.context.scene.render.pixel_aspect_x = 1
bpy.context.scene.render.pixel_aspect_y = 1
bpy.context.scene.render.fps = 24
bpy.context.scene.render.fps_base = 1



# SPDX-FileCopyrightText: 2023 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

from collections import namedtuple
from bpy.types import ShaderNodeMath, ShaderNodeMix


#################
# rl_outputs:
# list of outputs of Input Render Layer
# with attributes determining if pass is used,
# and MultiLayer EXR outputs names and corresponding render engines
#
# rl_outputs entry = (render_pass, rl_output_name, exr_output_name, in_eevee, in_cycles)
RL_entry = namedtuple('RL_Entry', ['render_pass', 'output_name', 'exr_output_name', 'in_eevee', 'in_cycles'])
rl_outputs = (
    RL_entry('use_pass_ambient_occlusion', 'Ambient Occlusion', 'Ambient Occlusion', True, True),
    RL_entry('use_pass_combined', 'Image', 'Combined', True, True),
    RL_entry('use_pass_diffuse_color', 'Diffuse Color', 'Diffuse Color', False, True),
    RL_entry('use_pass_diffuse_direct', 'Diffuse Direct', 'Diffuse Direct', False, True),
    RL_entry('use_pass_diffuse_indirect', 'Diffuse Indirect', 'Diffuse Indirect', False, True),
    RL_entry('use_pass_emit', 'Emission', 'Emission', False, True),
    RL_entry('use_pass_environment', 'Environment', 'Environment', False, False),
    RL_entry('use_pass_glossy_color', 'Glossy Color', 'Glossy Color', False, True),
    RL_entry('use_pass_glossy_direct', 'Glossy Direct', 'Glossy Direct', False, True),
    RL_entry('use_pass_glossy_indirect', 'Glossy Indirect', 'Glossy Indirect', False, True),
    RL_entry('use_pass_indirect', 'Indirect', 'Indirect', False, False),
    RL_entry('use_pass_material_index', 'Material Index', 'Material Index', False, True),
    RL_entry('use_pass_mist', 'Mist', 'Mist', True, True),
    RL_entry('use_pass_normal', 'Normal', 'Normal', True, True),
    RL_entry('use_pass_object_index', 'Object Index', 'Object Index', False, True),
    RL_entry('use_pass_shadow', 'Shadow', 'Shadow', False, True),
    RL_entry('use_pass_subsurface_color', 'Subsurface Color', 'Subsurface Color', True, True),
    RL_entry('use_pass_subsurface_direct', 'Subsurface Direct', 'Subsurface Direct', True, True),
    RL_entry('use_pass_subsurface_indirect', 'Subsurface Indirect', 'Subsurface Indirect', False, True),
    RL_entry('use_pass_transmission_color', 'Transmission Color', 'Transmission Color', False, True),
    RL_entry('use_pass_transmission_direct', 'Transmission Direct', 'Transmission Direct', False, True),
    RL_entry('use_pass_transmission_indirect', 'Transmission Indirect', 'Transmission Indirect', False, True),
    RL_entry('use_pass_uv', 'UV', 'UV', True, True),
    RL_entry('use_pass_vector', 'Speed', 'Vector', False, True),
    RL_entry('use_pass_z', 'Z', 'Depth', True, True),
)

# list of blend types of "Mix" nodes in a form that can be used as 'items' for EnumProperty.
# used list, not tuple for easy merging with other lists.
blend_types = [(enum.identifier, enum.name, enum.description)
               for enum in ShaderNodeMix.bl_rna.properties['blend_type'].enum_items_static]

# list of operations of "Math" nodes in a form that can be used as 'items' for EnumProperty.
# used list, not tuple for easy merging with other lists.
operations = [(enum.identifier, enum.name, enum.description)
              for enum in ShaderNodeMath.bl_rna.properties['operation'].enum_items_static]

# Operations used by the geometry boolean node and join geometry node
geo_combine_operations = [
    ('JOIN', 'Join Geometry', 'Join Geometry Mode'),
    ('INTERSECT', 'Intersect', 'Intersect Mode'),
    ('UNION', 'Union', 'Union Mode'),
    ('DIFFERENCE', 'Difference', 'Difference Mode'),
]

# in NWBatchChangeNodes additional types/operations. Can be used as 'items' for EnumProperty.
# used list, not tuple for easy merging with other lists.
navs = [
    ('CURRENT', 'Current', 'Leave at current state'),
    ('NEXT', 'Next', 'Next blend type/operation'),
    ('PREV', 'Prev', 'Previous blend type/operation'),
]

draw_color_sets = {
    "red_white": (
        (1.0, 1.0, 1.0, 0.7),
        (1.0, 0.0, 0.0, 0.7),
        (0.8, 0.2, 0.2, 1.0)
    ),
    "green": (
        (0.0, 0.0, 0.0, 1.0),
        (0.38, 0.77, 0.38, 1.0),
        (0.38, 0.77, 0.38, 1.0)
    ),
    "yellow": (
        (0.0, 0.0, 0.0, 1.0),
        (0.77, 0.77, 0.16, 1.0),
        (0.77, 0.77, 0.16, 1.0)
    ),
    "purple": (
        (0.0, 0.0, 0.0, 1.0),
        (0.38, 0.38, 0.77, 1.0),
        (0.38, 0.38, 0.77, 1.0)
    ),
    "grey": (
        (0.0, 0.0, 0.0, 1.0),
        (0.63, 0.63, 0.63, 1.0),
        (0.63, 0.63, 0.63, 1.0)
    ),
    "black": (
        (1.0, 1.0, 1.0, 0.7),
        (0.0, 0.0, 0.0, 0.7),
        (0.2, 0.2, 0.2, 1.0)
    )
}


def get_texture_node_types():
    return [
        "ShaderNodeTexBrick",
        "ShaderNodeTexChecker",
        "ShaderNodeTexEnvironment",
        "ShaderNodeTexGabor",
        "ShaderNodeTexGradient",
        "ShaderNodeTexIES",
        "ShaderNodeTexImage",
        "ShaderNodeTexMagic",
        "ShaderNodeTexMusgrave",
        "ShaderNodeTexNoise",
        "ShaderNodeTexSky",
        "ShaderNodeTexVoronoi",
        "ShaderNodeTexWave",
        "ShaderNodeTexWhiteNoise"
    ]


def nice_hotkey_name(punc):
    # convert the ugly string name into the actual character
    from bpy.types import KeyMapItem
    nice_name = {
        enum_item.identifier:
        enum_item.name
        for enum_item in KeyMapItem.bl_rna.properties['type'].enum_items
    }
    try:
        return nice_name[punc]
    except KeyError:
        return punc.replace("_", " ").title()



# SPDX-FileCopyrightText: 2026 Blender Authors
#
# SPDX-License-Identifier: GPL-2.0-or-later

from bpy.types import Operator


class RENDER_OT_swap_dimensions(Operator):
    bl_label = "Swap Dimensions"
    bl_idname = 'render.swap_dimensions'
    bl_description = "Flip X and Y resolutions"
    bl_options = {'INTERNAL'}

    def execute(self, context):
        rd = context.scene.render
        rd.resolution_x, rd.resolution_y = (rd.resolution_y, rd.resolution_x)

        return {'FINISHED'}


classes = (RENDER_OT_swap_dimensions,)



# SPDX-FileCopyrightText: 2018-2021 The glTF-Blender-IO authors
#
# SPDX-License-Identifier: Apache-2.0

import bpy

# Get compatibility at export with old files


def get_gltf_node_old_name():
    return "glTF Settings"

# Old group name


def get_gltf_old_group_node_name():
    return "glTF Metallic Roughness"


def get_gltf_node_name():
    return "glTF Material Output"


def create_settings_group(name):
    gltf_node_group = bpy.data.node_groups.new(name, 'ShaderNodeTree')

    # Oclusion (glTF Core)
    gltf_node_group.interface.new_socket("Occlusion", socket_type="NodeSocketFloat")

    # Thickness (glTF KHR_materials_volume)
    thicknessFactor = gltf_node_group.interface.new_socket("Thickness", socket_type="NodeSocketFloat", )
    thicknessFactor.default_value = 0.0

    # Dispersion (glTF KHR_materials_dispersion)
    dispersionFactor = gltf_node_group.interface.new_socket("Dispersion", socket_type="NodeSocketFloat", )
    dispersionFactor.default_value = 0.0

    # Iridescence (glTF KHR_materials_iridescence)
    iridescenceFactor = gltf_node_group.interface.new_socket("Iridescence Factor", socket_type="NodeSocketFloat", )
    iridescenceFactor.default_value = 0.0

    iridescenceTicknessMinimum = gltf_node_group.interface.new_socket(
        "Iridescence Thickness Minimum", socket_type="NodeSocketFloat", )
    iridescenceTicknessMinimum.default_value = 100.0

    gltf_node_group.nodes.new('NodeGroupOutput')
    gltf_node_group_input = gltf_node_group.nodes.new('NodeGroupInput')
    gltf_node_group_input.location = -200, 0
    return gltf_node_group
